# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=E0201
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Integration test cases for svnmailer.processes

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

from svnmailer_test import base, util
_tests = []

#### Test Bases                                     ####
########################################################
class TestBase(base.TestBase):
    """ Base class for processes tests """
    LOAD_MODULES = (('_proc', 'svnmailer.processes'),)


#### Test Cases                                     ####
########################################################
class ProcessTestCase(TestBase):
    """ svnmailer.processes.Process """
    PREFIX = 'Process'

    def setUp(self):
        """ set up """
        super(ProcessTestCase, self).setUp()
        self._pipe = None

    def tearDown(self):
        """ wait for the child """
        if self._pipe:
            self._pipe.wait()

        super(ProcessTestCase, self).tearDown()
 
    def test00pipe2(self):
        """ .pipe2 captures STDOUT """
        import sys
        command = [sys.executable, '-uc', 'import sys; sys.stdout.write("42")']
        self._pipe = self._proc.Process.pipe2(command)
        self.mock.result.append(self._pipe.fromchild.read())

        self.expect("42")

    def test01pipe2(self):
        """ .pipe2 pipes to STDIN """
        import sys
        command = [
            sys.executable, '-uc',
            'import sys; sys.stdout.write("wow %s" % sys.stdin.read());'
        ]
        self._pipe = self._proc.Process.pipe2(command)
        self._pipe.tochild.write("21\n42")
        self._pipe.tochild.close()
        self.mock.result.append(self._pipe.fromchild.read())

        self.expect("wow 21\n42")

    def test02pipe4(self):
        """ .pipe4 captures STDOUT """
        import sys
        command = [sys.executable, '-uc', 'import sys; sys.stdout.write("42")']
        self._pipe = self._proc.Process.pipe4(command)
        self.mock.result.append(self._pipe.fromchild.read())

        self.expect("42")

    def test03pipe4(self):
        """ .pipe4 captures STDERR """
        import sys
        command = [sys.executable, '-uc', 'import sys; sys.stderr.write("42")']
        self._pipe = self._proc.Process.pipe4(command)
        self.mock.result.append(self._pipe.fromchild.read())

        self.expect("42")

    def test04pipe4(self):
        """ .pipe4 captures STDOUT and STDERR """
        import sys
        command = [
            sys.executable, '-uc',
            r'import sys; sys.stdout.write("21\n"); '
            r'sys.stdout.flush(); sys.stderr.write("42");'
        ]
        self._pipe = self._proc.Process.pipe4(command)
        self.mock.result.append(self._pipe.fromchild.read())

        self.expect("21\n42")

    def test05pipe4(self):
        """ .pipe4 pipes to STDIN """
        import sys
        command = [
            sys.executable, '-uc',
            'import sys; sys.stdout.write("wow %s" % sys.stdin.read());'
        ]
        self._pipe = self._proc.Process.pipe4(command)
        self._pipe.tochild.write("21\n42")
        self._pipe.tochild.close()
        self.mock.result.append(self._pipe.fromchild.read())

        self.expect("wow 21\n42")

    def test06detach(self):
        """ .detach detaches correctly """
        testdata = util.findTestData()
        if not testdata:
            self.skip('testdata not found')
        if self._proc.Process.IMPLEMENTATION == 'dumb':
            self.skip('not implemented')

        import os, sys, tempfile
        script = os.path.join(testdata, "redirect.py")
        fd, stderr_file = tempfile.mkstemp()
        try:
            os.close(fd)
            cmd = [sys.executable, script, stderr_file]
            proc = self._proc.Process.pipe2(cmd)
            proc.tochild.write('this works!')
            proc.tochild.close()
            stdout = proc.fromchild.read()
            rcode = proc.wait()
            stderr = file(stderr_file, 'rb').read()
        finally:
            os.unlink(stderr_file)

        self.mock.result.extend([rcode, stdout, stderr])
        self.expect(0, "this works!", "!skrow siht")

_tests.append(ProcessTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
